%% ----- Step 4 -----
%{
  Initial release: 2021-04-18
  Updated:         2021-04-22

  Use the measurement PSI matrix and measured pressure to compute the
   spherical wave expansion function coefficients.
  For each maximum order (i.e. n = 0, 1, 2, ..., Nmax), we need to
   compute the expansion coefficients separately.
  The residuals (mean square error) returned by the least squares
   calculations is just one indicator of the fitting accuracy.
  Convergence of reconstructions at the CTA-2034 grid is also used as
   a separate (and more reliable) indicator of the fitting accuracy.
%}
% Start with reconstruction with order n = 0; pre-allocate storage for
%  arrays
n = 0;
residuals = zeros(1, Nmax+1); % Array to store fitting residuals
conv = zeros(1, Nmax); % Array to store convergence metric
p2034 = zeros(p2034_len, Nmax + 1); % Store CTA2034 reconstructions

% Solve for the coefficients using least sqaures: coeff = PSI \ press
tic;
[CD_vec, res] = lstsq_solve(PSI_mat(:, 1:2*(n+1)^2), p_meas(:, f_bin));
fprintf('N = %d, ', n)
fprintf('f = %.0f Hz, compute time = %.3f; \n', freqs(f_bin), toc)
residuals(1) = res;

% Reconstruct at the 2 plotting grids p1 & p2. The reconstruction
%  results are reshaped back to the same matrix dimensions as the
%  plotting grid coordinates for surface plotting.
p1 = PSI_p1(:, 1:2*(n+1)^2) * CD_vec;
p1 = reshape(p1, p1_shape);

p2 = PSI_p2(:, 1:2*(n+1)^2) * CD_vec;
p2 = reshape(p2, p2_shape);

% Reconstruct at the CTA2034 spinorama grid
p2034(:, 1) = PSI_2034(:, 1:2*(n+1)^2) * CD_vec;

% Setup the figure and axes, and plot the n = 0 reconstruction results
fig2 = figure('Name', 'Reconstructions', ...
              'Position', [400 140 1400 700]);

ax1 = subplot(2, 5, [1, 8]);
hold on;
x_ext = 1.7 * [-1 1];
y_ext = 1.7 * [-1 1];
z_ext = 1.7 * [-1 1];
line(ax1, x_ext, [0,0], [0,0], 'LineWidth', 1, 'Color', 'r');
line(ax1, [0,0], y_ext, [0,0], 'LineWidth', 1, 'Color', 'g');
line(ax1, [0,0], [0,0], z_ext, 'LineWidth', 1, 'Color', 'b');

% The first reconstruction plots
surf1 = surf(ax1, x_p1, y_p1, z_p1, abs(p1), 'FaceColor', 'interp');
surf1.EdgeColor = 'none';
surf2 = surf(ax1, x_p2, y_p2, z_p2, abs(p2), 'FaceColor', 'interp');
surf2.EdgeColor = 'none';

% If 'caxlims' is defined in 'params.mat', set colormap limits to
%  values of 'caxlims' in 'params.mat'.
% If 'cmap' is defined in 'params.mat', set colormap type to
%  value of 'cmap' in 'params.mat'.
if exist('params.mat', 'file')
    var_info = who('-file', 'params.mat');
    if ismember('caxlims', var_info)
        load('params.mat', 'caxlims');
        caxis(ax1, caxlims);
    end
    if ismember('cmap', var_info)
        load('params.mat', 'cmap');
        colormap(ax1, cmap);
    end
end
cbar = colorbar(ax1, 'eastoutside');

xlabel(ax1, 'X');        % label x axis
ylabel(ax1, 'Y');        % label y axis
zlabel(ax1, 'Z');        % label z axis
ax1.Title.String = sprintf('N = %d;  f = %.0f Hz\n', n, freqs(f_bin));
set(ax1, 'DataAspectRatio', [1 1 1]);
grid(ax1, 'on');
view(ax1, [130 20]);

ax2 = subplot(2, 5, [4, 10]);
hold off;

% Repeat reconstruction from n = 1 to Nmax and update plots
for n = 1:Nmax
    pause(0.5);
    tic;
    [CD_vec, res] = lstsq_solve(PSI_mat(:, 1:2*(n+1)^2), p_meas(:, f_bin));
    fprintf('N = %d, ', n)
    fprintf('f = %.0f Hz, compute time = %.3f; \n', freqs(f_bin), toc)
    residuals(n+1) = res;
    
    p1 = PSI_p1(:, 1:2*(n+1)^2) * CD_vec;
    p1 = reshape(p1, p1_shape);

    p2 = PSI_p2(:, 1:2*(n+1)^2) * CD_vec;
    p2 = reshape(p2, p2_shape);
    
    % Reconstruct at CTA2034 grid and compute convergence metrics
    p2034(:, n+1) = PSI_2034(:, 1:2*(n+1)^2) * CD_vec;
    conv(n) = norm(p2034(:, n) - p2034(:, n+1)) / p2034_len;
    
    % Update surface plots
    ax1.Title.String = sprintf('N = %d;  f = %.0f Hz\n', n, freqs(f_bin));
    surf1.CData = abs(p1);
    surf2.CData = abs(p2);
    
    % Plot least squares residuals and convergence metrics
    cla(ax2);
    semilogy(ax2, 1:n+1, residuals(1:n+1), 'bs-');
    hold(ax2, 'on');
    semilogy(ax2, 1:n, conv(1:n), 'rs-');
    ax2.Title.String = 'Fitting Residuals and Convergence Metrics';
    legend(ax2, 'Residuals', 'Convergence');
    xlabel(ax2, 'Expansion Order N');
    ylabel(ax2, 'Residuals/Convergence');
    grid(ax2, 'on');
    xlim(ax2, [0 Nmax]);
    ylim(ax2, [1e-4 1e2]);
    pbaspect(ax2, [1, 1, 1]);
    drawnow;
end